"""test for docs provider."""
# from tackle import tackle
#
#
# def test_provider_docs_provider_docs(change_dir):
#     """Run the provider docs."""
#     output = tackle('.')
#     assert output
#
#
# def test_provider_docs_provider_docs_requests(change_dir):
#     """Verify gcp get azs of region."""
#     output = tackle("requests.yaml", no_input=True)
#     assert output
